#include "graph.h"

extern vector<string> vUseTimeLabel;
extern vector<int>    vUseTimeData;

HWND CGraph::m_hWnd = NULL;

CGraph::CGraph(map<string, string> &mConf, int nWidth, int nHeight, const vector<string> &vLabel, const vector<int> &vData) :
m_nWidth(nWidth), m_nHeight(nHeight), m_vLabel(vLabel), m_vData(vData) {
	m_strTitle = mConf["title"];
	m_hbmpBackground = LoadImageEx(strPath + mConf["background.filename"]);
	m_cBackground = GetColor(mConf, "background.color", 0xffffff);
	m_cFrame = GetColor(mConf, "frame.color", 0x000000);
	m_cLine  = GetColor(mConf, "line.color", 0x000000);
	m_cFont  = GetColor(mConf, "font.color", 0x000000);
	m_nFrameWidth = GetNumber(mConf, "frame.width", 2);
	m_nLineWidth  = GetNumber(mConf, "line.width", 2);
	
	for (int i = 0; i < 3; ++i) {
		m_ahbmpMarkImage[i] = LoadImageEx(strPath + mConf["mark" + ToStr(i) + ".filename"]);
		if (m_ahbmpMarkImage[i])
			m_ahbmpMarkMask[i] = MakeMaskBitmap(m_ahbmpMarkImage[i]);
	}
	
	RECT rcClient;
	GetClientRect(m_hWnd, &rcClient);
	if (rcClient.right != nWidth || rcClient.bottom != nHeight || !IsWindowVisible(m_hWnd)) {
		// TCYݒ
		RECT rcScreen, rcWindow = { 0, 0, nWidth, nHeight };
		GetWindowRect(GetDesktopWindow(), &rcScreen);
		AdjustWindowRectEx(&rcWindow, GetWindowLong(m_hWnd, GWL_STYLE), false, GetWindowLong(m_hWnd, GWL_EXSTYLE));
		int width = rcWindow.right - rcWindow.left;
		int height = rcWindow.bottom - rcWindow.top;
		MoveWindow(m_hWnd, (rcScreen.right - width) / 2, (rcScreen.bottom - height) / 2, width, height, false);
	}
}

CGraph::~CGraph() {
	if (m_hbmpBackground)
		DeleteObject(m_hbmpBackground);
	for (int i = 0; i < 3; ++i) {
		if (m_ahbmpMarkImage[i])
			DeleteObject(m_ahbmpMarkImage[i]);
		if (m_ahbmpMarkMask[i])
			DeleteObject(m_ahbmpMarkMask[i]);
	}
	
}

CGraph *CGraph::CreateGraph(map<string, string> &mConf, const vector<string> &vLabel, const vector<int> &vData) {
	string &strGraph = mConf["graph"];
	
	if (stricmp(strGraph.c_str(), "line") == 0)
		return new CLineGraph(mConf, vLabel, vData);
	
	else if (stricmp(strGraph.c_str(), "bar") == 0)
		return new CVBarGraph(mConf, vLabel, vData);
	
	else if (stricmp(strGraph.c_str(), "radar") == 0)
		return new CAIGraph(mConf, vLabel, vData);
	
	else if (stricmp(strGraph.c_str(), "radar2") == 0)
		return new CRadarChart(mConf, vLabel, vData);
	
	else
		return new CUseTimeGraph(mConf);
}

void CGraph::DrawBackground(HDC hdc) {
	{ // wi
		RECT rc = { 0, 0, m_nWidth, m_nHeight };
		HBRUSH hBrush = CreateSolidBrush(m_cBackground);
		FillRect(hdc, &rc, hBrush);
		DeleteObject(hBrush);
	}
	
	if (m_hbmpBackground) { // wi摜
		BITMAP bm;
		GetObject(m_hbmpBackground, sizeof bm, &bm);
		HDC hdcBmp = CreateCompatibleDC(hdc);
		HGDIOBJ hbmpOld = SelectObject(hdcBmp, m_hbmpBackground);
		BitBlt(hdc, 0, 0, m_nWidth, m_nHeight, hdcBmp, (bm.bmWidth - m_nWidth) / 2, (bm.bmHeight - m_nHeight) / 2, SRCCOPY);
		SelectObject(hdcBmp, hbmpOld);
		DeleteObject(hdcBmp);
	}
}


// ܂Ot
CLineGraph::CLineGraph(map<string, string> &mConf, const vector<string> &vLabel, const vector<int> &vData) : CGraph(mConf, 450, 340, vLabel, vData) {}
CLineGraph::~CLineGraph() {}

void CLineGraph::OnDraw(HDC hdc) {
	vector<int> vPos;
	for (int i = 0; i < m_vLabel.size(); ++i)
		vPos.push_back(80 + (400 - 80) * i / (m_vLabel.size() - 1));
	
	// `
	DrawBackground(hdc);
	
	if (0 < m_nFrameWidth) { // Otg
		HGDIOBJ hOldPen = SelectObject(hdc, CreatePen(PS_SOLID, m_nFrameWidth, m_cFrame));
		MoveToEx(hdc, 55, 50, NULL);
		LineTo(hdc, 55, 260);
		LineTo(hdc, 420, 260);
		DeleteObject(SelectObject(hdc, hOldPen));
	}
	
	{ // ^Cg
		LOGFONT lf = { 0 };
		lf.lfHeight = -(12 * GetDeviceCaps(hdc, LOGPIXELSY) / 72);
		lf.lfWeight = FW_BOLD;
		lf.lfCharSet = SHIFTJIS_CHARSET;
		lf.lfEscapement = 2700;
		strcpy(lf.lfFaceName, "@lr oSVbN");
		HGDIOBJ hFontOld = SelectObject(hdc, CreateFontIndirect(&lf));
		SetTextColor(hdc, m_cFont);
		SetTextAlign(hdc, TA_CENTER | TA_BOTTOM);
		TextOut(hdc, 20, 155, m_strTitle.c_str(), m_strTitle.length());
		SetTextAlign(hdc, TA_LEFT | TA_TOP);
		DeleteObject(SelectObject(hdc, hFontOld));
	}
	
	if (!m_vLabel.empty()) { // x
		LOGFONT lf = { 0 };
		lf.lfHeight = -(12 * GetDeviceCaps(hdc, LOGPIXELSY) / 72);
		lf.lfWeight = FW_BOLD;
		lf.lfCharSet = SHIFTJIS_CHARSET;
		HGDIOBJ hFontOld = SelectObject(hdc, CreateFontIndirect(&lf));
		SetTextColor(hdc, m_cFont);
		SetTextAlign(hdc, TA_CENTER | TA_TOP);
		for (int i = 0; i < m_vLabel.size(); ++i)
			TextOut(hdc, vPos[i], 270, m_vLabel[i].c_str(), m_vLabel[i].length());
		SetTextAlign(hdc, TA_LEFT | TA_TOP);
		DeleteObject(SelectObject(hdc, hFontOld));
	}
	
	if (!m_vData.empty() && 0 < m_nLineWidth) { // Ot
		HGDIOBJ hOldPen = SelectObject(hdc, CreatePen(PS_SOLID, m_nLineWidth, m_cLine));
		MoveToEx(hdc, vPos[0], 230 - m_vData[0], NULL);
		for (int i = 0; i < m_vData.size(); ++i) {
			LineTo(hdc, vPos[i], 230 - m_vData[i]);
		}
		DeleteObject(SelectObject(hdc, hOldPen));
	}
	
	if (!m_vData.empty()) { // Ot
		HDC hdcBmp = CreateCompatibleDC(hdc);
		int nMax = *max_element(m_vData.begin(), m_vData.end());
		for (int i = 0; i < m_vData.size(); ++i) {
			int nMarkNo;
			if (m_vData[i] == 0)
				nMarkNo = 0;
			else if (m_vData[i] == nMax)
				nMarkNo = 2;
			else
				nMarkNo = 1;
			int x = vPos[i];
			int y = 230 - m_vData[i];
			if (m_ahbmpMarkImage[nMarkNo]) {
				BITMAP bm;
				GetObject(m_ahbmpMarkImage[nMarkNo], sizeof bm, &bm);
				HGDIOBJ hbmpOld = SelectObject(hdcBmp, m_ahbmpMarkMask[nMarkNo]);
				BitBlt(hdc, x - bm.bmWidth / 2, y - bm.bmHeight / 2, bm.bmWidth, bm.bmHeight, hdcBmp, 0, 0, MERGEPAINT);
				SelectObject(hdcBmp, m_ahbmpMarkImage[nMarkNo]);
				BitBlt(hdc, x - bm.bmWidth / 2, y - bm.bmHeight / 2, bm.bmWidth, bm.bmHeight, hdcBmp, 0, 0, SRCAND);
				SelectObject(hdcBmp, hbmpOld);
			}
		}
		DeleteDC(hdcBmp);
	}
}

// c_Ot
CVBarGraph::CVBarGraph(map<string, string> &mConf, const vector<string> &vLabel, const vector<int> &vData)
: CGraph(mConf, 450, 340, vLabel, vData) {
	m_nBarWidth = GetNumber(mConf, "bar.width", 16);
	m_cBar = GetColor(mConf, "bar.color", 0xffffffff);
}
CVBarGraph::~CVBarGraph() {}

void CVBarGraph::OnDraw(HDC hdc) {
	vector<int> vPos;
	for (int i = 0; i < m_vLabel.size(); ++i)
		vPos.push_back(100 + (380 - 100) * i / (m_vLabel.size() - 1));
	
	// `
	DrawBackground(hdc);
	
	{ // ^Cg
		LOGFONT lf = { 0 };
		lf.lfHeight = -(12 * GetDeviceCaps(hdc, LOGPIXELSY) / 72);
		lf.lfWeight = FW_BOLD;
		lf.lfCharSet = SHIFTJIS_CHARSET;
		lf.lfEscapement = 2700;
		strcpy(lf.lfFaceName, "@lr oSVbN");
		HGDIOBJ hFontOld = SelectObject(hdc, CreateFontIndirect(&lf));
		SetTextColor(hdc, m_cFont);
		SetTextAlign(hdc, TA_CENTER | TA_BOTTOM);
		TextOut(hdc, 20, 155, m_strTitle.c_str(), m_strTitle.length());
		SetTextAlign(hdc, TA_LEFT | TA_TOP);
		DeleteObject(SelectObject(hdc, hFontOld));
	}
	
	if (!m_vLabel.empty()) { // x
		LOGFONT lf = { 0 };
		lf.lfHeight = -(12 * GetDeviceCaps(hdc, LOGPIXELSY) / 72);
		lf.lfWeight = FW_BOLD;
		lf.lfCharSet = SHIFTJIS_CHARSET;
		HGDIOBJ hFontOld = SelectObject(hdc, CreateFontIndirect(&lf));
		SetTextColor(hdc, m_cFont);
		SetTextAlign(hdc, TA_CENTER | TA_TOP);
		for (int i = 0; i < m_vLabel.size(); ++i)
			TextOut(hdc, vPos[i], 270, m_vLabel[i].c_str(), m_vLabel[i].length());
		SetTextAlign(hdc, TA_LEFT | TA_TOP);
		DeleteObject(SelectObject(hdc, hFontOld));
	}
	
	if (!m_vData.empty()) { // Ot_
		HGDIOBJ hOldPen = NULL;
		if (0 < m_nLineWidth)
			hOldPen = SelectObject(hdc, CreatePen(PS_SOLID, m_nLineWidth, m_cLine));
		HGDIOBJ hOldBrush = NULL;
		if (m_cBar != 0xffffffff)
			hOldBrush = SelectObject(hdc, CreateSolidBrush(m_cBar));
		for (int i = 0; i < m_vData.size(); ++i) {
			if (m_cBar != 0xffffffff)
				Rectangle(hdc, vPos[i] - m_nBarWidth / 2, 260 - m_vData[i], vPos[i] + m_nBarWidth / 2, 260);
			if (0 < m_nLineWidth) {
				MoveToEx(hdc, vPos[i] - m_nBarWidth / 2, 260, NULL); // 
				LineTo(  hdc, vPos[i] - m_nBarWidth / 2, 260 - m_vData[i]); // 
				LineTo(  hdc, vPos[i] + m_nBarWidth / 2, 260 - m_vData[i]); // E
				LineTo(  hdc, vPos[i] + m_nBarWidth / 2, 260); // E
			}
		}
		if (0 < m_nLineWidth)
			DeleteObject(SelectObject(hdc, hOldPen));
		if (m_cBar != 0xffffffff)
			DeleteObject(SelectObject(hdc, hOldBrush));
	}
	
	if (0 < m_nFrameWidth) { // Otg
		HGDIOBJ hOldPen = SelectObject(hdc, CreatePen(PS_SOLID, m_nFrameWidth, m_cFrame));
		MoveToEx(hdc, 55, 50, NULL);
		LineTo(hdc, 55, 260);
		LineTo(hdc, 420, 260);
		DeleteObject(SelectObject(hdc, hOldPen));
	}
	
	if (!m_vData.empty()) { // Ot
		HDC hdcBmp = CreateCompatibleDC(hdc);
		int nMax = *max_element(m_vData.begin(), m_vData.end());
		for (int i = 0; i < m_vData.size(); ++i) {
			int nMarkNo;
			if (m_vData[i] == 0)
				nMarkNo = 0;
			else if (m_vData[i] == nMax)
				nMarkNo = 2;
			else
				nMarkNo = 1;
			int x = vPos[i];
			int y = 230 - m_vData[i];
			if (m_ahbmpMarkImage[nMarkNo]) {
				BITMAP bm;
				GetObject(m_ahbmpMarkImage[nMarkNo], sizeof bm, &bm);
				HGDIOBJ hbmpOld = SelectObject(hdcBmp, m_ahbmpMarkMask[nMarkNo]);
				BitBlt(hdc, x - bm.bmWidth / 2, y - bm.bmHeight / 2, bm.bmWidth, bm.bmHeight, hdcBmp, 0, 0, MERGEPAINT);
				SelectObject(hdcBmp, m_ahbmpMarkImage[nMarkNo]);
				BitBlt(hdc, x - bm.bmWidth / 2, y - bm.bmHeight / 2, bm.bmWidth, bm.bmHeight, hdcBmp, 0, 0, SRCAND);
				SelectObject(hdcBmp, hbmpOld);
			}
		}
		DeleteDC(hdcBmp);
	}
}


// ɂ܂ԃOt
CUseTimeGraph::CUseTimeGraph(map<string, string> &mConf) : CLineGraph(mConf, vUseTimeLabel, vUseTimeData)
{
	m_nMaxNum = GetNumber(mConf, "max", 24*60); //Pʁ
	UpdateUseTime();
}

CUseTimeGraph::~CUseTimeGraph() {}

void CUseTimeGraph::UpdateUseTime(void)
{
	for ( int i = 0 ; i < 7 ; ++i ) {
		m_vData[i] = vUseTimeData[i] * m_nMaxNum / (24*60);
		if ( m_vData[i] > m_nMaxNum ) {
			m_vData[i] = m_nMaxNum;
		}
	}
}

void CUseTimeGraph::OnUpdate(void)
{
	if (vUseTimeData[6] != m_vData[6]) {
		if (vUseTimeData[5] != m_vData[5])
			m_vLabel = vUseTimeLabel;

		UpdateUseTime();

		InvalidateRect(m_hWnd, NULL, TRUE);
		UpdateWindow(m_hWnd);
	}
}


// [_[`[g
CRadarChart::CRadarChart(map<string, string> &mConf, const vector<string> &vLabel, const vector<int> &vData) : CGraph(mConf, 288, 288, vLabel, vData) {}
CRadarChart::~CRadarChart() {}

void CRadarChart::OnDraw(HDC hdc) {
	vector<int> vPosX, vPosY;
	for (int i = 0; i < m_vLabel.size(); ++i) {
		vPosX.push_back(m_nWidth  / 2 + (int)(m_vData[i] *  cos(3.1415926535 / 2 - 2 * 3.1415926535 / m_vData.size() * i)));
		vPosY.push_back(m_nHeight / 2 + (int)(m_vData[i] * -sin(3.1415926535 / 2 - 2 * 3.1415926535 / m_vData.size() * i)));
	}
	
	// `
	DrawBackground(hdc);
	
	if (0 < m_nFrameWidth) { // Otg
		OnDrawFrame(hdc);
	}
	
	{ // ^Cg
		LOGFONT lf = { 0 };
		lf.lfHeight = -(12 * GetDeviceCaps(hdc, LOGPIXELSY) / 72);
		lf.lfWeight = FW_BOLD;
		lf.lfCharSet = SHIFTJIS_CHARSET;
		strcpy(lf.lfFaceName, "lr oSVbN");
		HGDIOBJ hFontOld = SelectObject(hdc, CreateFontIndirect(&lf));
		SetTextColor(hdc, m_cFont);
		TextOut(hdc, 10, 10, m_strTitle.c_str(), m_strTitle.length());
		DeleteObject(SelectObject(hdc, hFontOld));
	}
	
	if (!m_vData.empty() && 0 < m_nLineWidth) { // Ot
		HGDIOBJ hOldPen = SelectObject(hdc, CreatePen(PS_SOLID, m_nLineWidth, m_cLine));
		MoveToEx(hdc, vPosX[0], vPosY[0], NULL);
		for (int i = 0; i < m_vData.size(); ++i)
			LineTo(hdc, vPosX[i], vPosY[i]);
		LineTo(hdc, vPosX[0], vPosY[0]);
		DeleteObject(SelectObject(hdc, hOldPen));
	}
	
	if (!m_vData.empty()) { // Ot
		HDC hdcBmp = CreateCompatibleDC(hdc);
		int nMax = *max_element(m_vData.begin(), m_vData.end());
		for (int i = 0; i < m_vData.size(); ++i) {
			int nMarkNo;
			if (m_vData[i] == 0)
				nMarkNo = 0;
			else if (m_vData[i] == nMax)
				nMarkNo = 2;
			else
				nMarkNo = 1;
			if (m_ahbmpMarkImage[nMarkNo]) {
				BITMAP bm;
				GetObject(m_ahbmpMarkImage[nMarkNo], sizeof bm, &bm);
				HGDIOBJ hbmpOld = SelectObject(hdcBmp, m_ahbmpMarkMask[nMarkNo]);
				BitBlt(hdc, vPosX[i] - bm.bmWidth / 2, vPosY[i] - bm.bmHeight / 2, bm.bmWidth, bm.bmHeight, hdcBmp, 0, 0, MERGEPAINT);
				SelectObject(hdcBmp, m_ahbmpMarkImage[nMarkNo]);
				BitBlt(hdc, vPosX[i] - bm.bmWidth / 2, vPosY[i] - bm.bmHeight / 2, bm.bmWidth, bm.bmHeight, hdcBmp, 0, 0, SRCAND);
				SelectObject(hdcBmp, hbmpOld);
			}
		}
		DeleteDC(hdcBmp);
	}
	
	if (!m_vLabel.empty()) { // Otx
		LOGFONT lf = { 0 };
		lf.lfHeight = -(10 * GetDeviceCaps(hdc, LOGPIXELSY) / 72);
		lf.lfCharSet = SHIFTJIS_CHARSET;
		HGDIOBJ hFontOld = SelectObject(hdc, CreateFontIndirect(&lf));
		SetTextColor(hdc, m_cFont);
		for (int i = 0; i < m_vData.size(); ++i)
			TextOut(hdc, vPosX[i], vPosY[i] + 10, m_vLabel[i].c_str(), m_vLabel[i].length());
		DeleteObject(SelectObject(hdc, hFontOld));
	}
}

void CRadarChart::OnDrawFrame(HDC hdc) {
	HGDIOBJ hOldPen = SelectObject(hdc, CreatePen(PS_SOLID, m_nFrameWidth, m_cFrame));
	for (int i = 0; i < m_vData.size(); ++i) {
		MoveToEx(hdc, m_nWidth / 2, m_nHeight / 2, NULL);
		LineTo(hdc, m_nWidth  / 2 + (m_nWidth  / 2 - 30) *  cos(3.1415926535 / 2 - 2 * 3.1415926535 / m_vData.size() * i),
			m_nHeight / 2 + (m_nHeight / 2 - 30) * -sin(3.1415926535 / 2 - 2 * 3.1415926535 / m_vData.size() * i));
	}
	DeleteObject(SelectObject(hdc, hOldPen));
}

// [_[`[gAIOt
CAIGraph::CAIGraph(map<string, string> &mConf, const vector<string> &vLabel, const vector<int> &vData) : CRadarChart(mConf, vLabel, vData) {}
CAIGraph::~CAIGraph() {}

void CAIGraph::OnDrawFrame(HDC hdc) {
	HGDIOBJ hOldPen = SelectObject(hdc, CreatePen(PS_SOLID, m_nFrameWidth, m_cFrame));
	MoveToEx(hdc, 30, 288 / 2, NULL);
	LineTo(hdc, 288 - 30, 288 / 2);
	MoveToEx(hdc, 288 / 2, 30, NULL);
	LineTo(hdc, 288 / 2, 288 - 30);
	DeleteObject(SelectObject(hdc, hOldPen));
}
